/*
 *
 * Copyright (C) 2004 Texas Instruments.
 * Copyright (C) 2007 SpinetiX S.A.
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------------
 * Modifications:
 * ver. 1.0: Oct 2005, Swaminathan S
 * 2007 - Added support for Bonsai, Diego Santa Cruz
 * August 2007 - Modification for VICP init  Nageswari S 
 *
 */

#include <common.h>
#include <i2c.h>
#include "timer.h"
#include <status_led.h>
#include <asm/byteorder.h>
#include <net.h>

#if (CONFIG_COMMANDS & CFG_CMD_NAND)
#include <linux/mtd/nand.h>
extern struct nand_chip nand_dev_desc[CFG_MAX_NAND_DEVICE];
#endif

/* The registered machine number for Bonsai,
   from http://www.arm.linux.org.uk/developer/machines/download.php */
#define MACHINE_NUMBER 1455

/* Hardware info */

#define EEPROM_CORE_I2C_ADDR		0x52
#define EEPROM_CORE_I2C_ADDR_OLD	0x50
#define EEPROM_CARRIER_I2C_ADDR		0x53
#define HW_INFO_SIZE			0x28
#define ETH_ADDR_OFFSET			0x20
static unsigned char core_hw_info[HW_INFO_SIZE];
static unsigned char carrier_hw_info[HW_INFO_SIZE];
 /* 1 if OK, -1 if not set, 0 if not valid or not yet read */
static int core_hw_info_ok;
static int carrier_hw_info_ok;
/* Hardware revision number. High 16 bits are major, lower 16 bits are minor */
static u32 core_hw_rev;
static u32 carrier_hw_rev;
#define MK_HW_REV(major, minor)					\
		( ( ( (major) & 0xFFFF ) << 16 ) | ( (minor) & 0xFFFF) )
#define HW_REV_MAJOR(rev)	( ( (rev) >> 16 ) & 0xFFFF )
#define HW_REV_MINOR(rev)	( (rev) & 0xFFFF )
static struct tag_serialnr board_serial_nr;

static void load_hw_info(void);

/* Flag to signal that we could not reset the I2C bus */
static int i2c_bus_failed;
static int i2c_bus_count;

#if 0
void flash__init (void);
void ether__init (void);
#endif
#define PLL1_PLLM   *(volatile unsigned int *)0x01c40910
#define PLL1_DIV1   *(volatile unsigned char *)0x01c40918
#define PLL1_DIV2   *(volatile unsigned char *)0x01c4091C
#define PLL1_DIV3   *(volatile unsigned char *)0x01c40920
#define PLL1_DIV5   *(volatile unsigned char *)0x01c40964
#define PLL2_PLLM   *(volatile unsigned int *)0x01c40D10
#define PLL2_DIV1   *(volatile unsigned char *)0x01c40D18
#define PLL2_DIV2   *(volatile unsigned char *)0x01c40D1C

#define GPIO_PSHBTN_BANK	GPIOBK01
#define GPIO_PSHBTN_BIT		(1 << 9)

#define PINMUX4_I2CEN		(1 << 7)
#define GPIO_I2C_SCL_BIT	(1 << 11)
#define GPIO_I2C_SDA_BIT	(1 << 12)

/* DDC muxer controlled by GPIO 1 */
#define GPIO_DDC_DISABLE_BANK	GPIOBK01
#define GPIO_DDC_DISABLE_BIT	(1 << 1)

/* Peripheral reset is GPIO 45 */
#define GPIO_PERIPH_RESET_BANK	GPIOBK23
#define GPIO_PERIPH_RESET_BIT	(1 << 13)

/* USB power is GPIO 49 */
#define GPIO_USBPWR_BANK	GPIOBK23
#define GPIO_USBPWR_BIT		(1 << 17)

/* -------------------------------------------------------------------------- *
 *    GPIO Register structure - See sprue25.pdf for more details.
 * -------------------------------------------------------------------------- */

typedef struct _gpio_regs_  {
    volatile unsigned int PID;           /* 0h Peripheral Identification Register Section 3.1 */
    volatile unsigned char reserved0[4];
    volatile unsigned int BINTEN;        /* 8h GPIO Interrupt Per-Bank Enable Register Section 3.2 */
} gpioRegs;

typedef struct _gpiobank_regs_  {
    volatile unsigned int DIR;        /* 10h GPIO Banks 0 and 1 Direction Register Section 3.3 */
    volatile unsigned int OUT;        /* 14h DATA01 GPIO Banks 0 and 1 Output Data Register Section 3.4 */
    volatile unsigned int SET;        /* 18h DATA01 GPIO Banks 0 and 1 Set Data Register Section 3.5 */
    volatile unsigned int CLR;        /* 1Ch DATA01 GPIO Banks 0 and 1 Clear Data Register Section 3.6 */
    volatile unsigned int IN;         /* 20h DATA01 GPIO Banks 0 and 1 Input Data Register Section 3.7 */
    volatile unsigned int SETR;       /* 24h RIS_TRIG01 GPIO Banks 0 and 1 Set Rising Edge Interrupt Register Section 3.8 */
    volatile unsigned int CLRR;       /* 28h RIS_TRIG01 GPIO Banks 0 and 1 Clear Rising Edge Interrupt Register Section 3.9 */
    volatile unsigned int SETF;       /* 2Ch FAL_TRIG01 GPIO Banks 0 and 1 Set Falling Edge Interrupt Register Section 3.10 */
    volatile unsigned int CLRF;       /* 30h FAL_TRIG01 GPIO Banks 0 and 1 Clear Falling Edge Interrupt Register Section 3.11 */
    volatile unsigned int INTSTAT;    /* 34h GPIO Banks 0 and 1 Interrupt Status Register Section 3.12 */
} gpioBanks;

#define GPIO ((gpioRegs*) 0x01c67000)
#define GPIOBK01 ((gpioBanks*) 0x01c67010)
#define GPIOBK23 ((gpioBanks*) 0x01c67038)
#define GPIOBK4  ((gpioBanks*) 0x01c67060)

static void disable_i2c_ddc(void);
static void reset_i2c_scl(void);
static void reset_peripherals(void);

/*******************************************
 Routine: is_pushbtn_pressed()
 Description:  tests the current status of the push button
*******************************************/
static int is_pushbtn_pressed(void)
{
	/* Carriers up to ver 2.1 have no push button */
	if (carrier_hw_rev != 0 && carrier_hw_rev <= MK_HW_REV(2,1))
		return 0;
	return ( (GPIO_PSHBTN_BANK->IN & GPIO_PSHBTN_BIT) == 0);
}

/*******************************************
 Routine: check_pushbtn()
 Description:  checks if the push button has been pressed for
 the specified time, in msec. Returns zero if not, non-zero otherwise.
 The time is cumulative between successive calls.
*******************************************/
static int check_pushbtn(unsigned int min_msecs)
{
	static unsigned int total_msecs;

	/* Loop in steps of 1 msec. */
	while (total_msecs < min_msecs) {
		udelay(1000);
		total_msecs++;
		if ( ! is_pushbtn_pressed() ) {
			return 0;
		}
	}
	return 1;
}

/*******************************************
 Routine: get_board_rev()
 Description:  Returns the board rev number, taking the carrier
 number as board number. If no carrier HW rev has been set in
 EEPROM yet it takes the value of the 'boardrev' environment variable.
*******************************************/
u32 get_board_rev(void)
{
	const char *s;
	if ( carrier_hw_info_ok < 0 && (s = getenv("boardrev")) ) {
		char *p;
		u32 rev = simple_strtoul(s, &p, 10);
		if (*p != '\0' || *s == '\0') {
			rev = 0;
			printf("ERROR: invalid board revision number "
			       "in 'boardrev'\n");
		}
		return rev;
	} else {
		return carrier_hw_rev;
	}
}

/*******************************************
 Routine: get_board_serial()
 Description:  Returns the board serial number,
 taking the carrier MAC address as board number.
*******************************************/
void get_board_serial(struct tag_serialnr *serialnr)
{
	*serialnr = board_serial_nr;
}


/* PSC Domains */
#define LPSC_VPSSMSTR       0       // VPSS Master LPSC
#define LPSC_VPSSSLV        1       // VPSS Slave LPSC
#define LPSC_TPCC           2       // TPCC LPSC
#define LPSC_TPTC0          3       // TPTC0 LPSC
#define LPSC_TPTC1          4       // TPTC1 LPSC
#define LPSC_EMAC           5       // EMAC LPSC
#define LPSC_EMAC_WRAPPER   6       // EMAC WRAPPER LPSC
#define LPSC_MDIO           7       // MDIO LPSC
#define LPSC_IEEE1394       8       // IEEE1394 LPSC
#define LPSC_USB            9       // USB LPSC
#define LPSC_ATA            10      // ATA LPSC
#define LPSC_VLYNQ          11      // VLYNQ LPSC
#define LPSC_UHPI           12      // UHPI LPSC
#define LPSC_DDR_EMIF       13      // DDR_EMIF LPSC
#define LPSC_AEMIF          14      // AEMIF LPSC
#define LPSC_MMC_SD         15      // MMC_SD LPSC
#define LPSC_MEMSTICK       16      // MEMSTICK LPSC
#define LPSC_McBSP          17      // McBSP LPSC
#define LPSC_I2C            18      // I2C LPSC
#define LPSC_UART0          19      // UART0 LPSC
#define LPSC_UART1          20      // UART1 LPSC
#define LPSC_UART2          21      // UART2 LPSC
#define LPSC_SPI            22      // SPI LPSC
#define LPSC_PWM0           23      // PWM0 LPSC
#define LPSC_PWM1           24      // PWM1 LPSC
#define LPSC_PWM2           25      // PWM2 LPSC
#define LPSC_GPIO           26      // GPIO LPSC
#define LPSC_TIMER0         27      // TIMER0 LPSC
#define LPSC_TIMER1         28      // TIMER1 LPSC
#define LPSC_TIMER2         29      // TIMER2 LPSC
#define LPSC_SYSTEM_SUBSYS  30      // SYSTEM SUBSYSTEM LPSC
#define LPSC_ARM            31      // ARM LPSC
#define LPSC_SCR2           32      // SCR2 LPSC
#define LPSC_SCR3           33      // SCR3 LPSC
#define LPSC_SCR4           34      // SCR4 LPSC
#define LPSC_CROSSBAR       35      // CROSSBAR LPSC
#define LPSC_CFG27          36      // CFG27 LPSC
#define LPSC_CFG3           37      // CFG3 LPSC
#define LPSC_CFG5           38      // CFG5 LPSC
#define LPSC_GEM            39      // GEM LPSC
#define LPSC_IMCOP          40      // IMCOP LPSC

#define CHP_SHRTSW          *( volatile unsigned int* )( 0x01C40038 )   
#define GBLCTL              *( volatile unsigned int* )( 0x01C41010 ) 
#define EPCPR               *( volatile unsigned int* )( 0x01C41070 )
#define EPCCR               *( volatile unsigned int* )( 0x01C41078 )
#define PDSTAT              *( volatile unsigned int* )( 0x01C41200 )
#define PDSTAT1             *( volatile unsigned int* )( 0x01C41204 )
#define PDCTL               *( volatile unsigned int* )( 0x01C41300 )
#define PDCTL1              *( volatile unsigned int* )( 0x01C41304 )
#define VBPR                *( volatile unsigned int* )( 0x20000020 )

#define PSC_ADDR            0x01C41000
#define PTCMD               (PSC_ADDR+0x120)
#define PTSTAT              (PSC_ADDR+0x128)
#define UARTPWREMU_MGMT     0x01c20030
#define VDD3P3V_PWDN        0x01c40048
#define PINMUX0		    0x01C40000
#define PINMUX4		    0x01C40004


void davinci_psc_all_enable(void);

/*******************************************
 Routine: delay
 Description:  Delay function
*******************************************/
static inline void delay (unsigned long loops)
{
	       __asm__ volatile ("1:\n"
		     "subs %0, %1, #1\n"
		     "bne 1b":"=r" (loops):"0" (loops));
}

/*******************************************
 Routine: board_init
 Description:  Board Initialization routine
*******************************************/
int board_init (void)
{
    	  DECLARE_GLOBAL_DATA_PTR;

        /* arch number of Bonsai DaVinci Board */
	gd->bd->bi_arch_number = MACHINE_NUMBER;

      	/* adress of boot parameters */
       	gd->bd->bi_boot_params = LINUX_BOOT_PARAM_ADDR;

      /* Power on required peripherals */
    	davinci_psc_all_enable();
#if 0
      	/**
	 * This speeds up your boot a quite a bit.  However to make it
 	 * work, you need make sure your kernel startup flush bug is fixed.
	 */
	icache_enable ();
#endif

	eth_initialize(gd->bd);

      	inittimer ();

	disable_i2c_ddc(); /* must be called after setting the timers */

	reset_i2c_scl(); /* must be called after setting the timers */

	reset_peripherals();

#ifdef CONFIG_SILENT_CONSOLE
	/* start in silent mode, this is too early in init to reliably
	 * check and report the board HW revision and do more advanced
	 * initialization, we will reset this later */
	gd->flags |= GD_FLG_SILENT;
#endif
	
      	return 0;
}


/**************************************
 Routine: board_setup_psc_on
 Description:  Enable a PSC domain
**************************************/
void board_setup_psc_on( unsigned int domain, unsigned int id )
{
        volatile unsigned int* mdstat
		= ( unsigned int* )( 0x01C41800 + 4 * id );
	volatile unsigned int* mdctl
		= ( unsigned int* )( 0x01C41A00 + 4 * id );

	// Set PowerDomain to turn on
	*mdctl |= 0x00000003;

    	  if ( ( PDSTAT & 0x00000001 ) == 0 )
    	  {
                PDCTL1 |= 0x1;
        	      *(volatile unsigned int*)PTCMD = ( 1 << domain );
        	      while ( ( ( ( EPCPR >> domain ) & 1 ) == 0 ) );

        	      PDCTL1 |= 0x100 ;
        	      while(!(((*(volatile unsigned int*)PTSTAT >> domain) & 1) == 0));
    	  }
    	  else
    	  {
        	      *(volatile unsigned int*)PTCMD = ( 1<<domain );
        	      while(!(((*(volatile unsigned int*)PTSTAT >> domain) & 1) == 0));
    	  }

    	  while( ! ( ( *mdstat & 0x0000001F ) == 0x3 ) );
}

/**************************************
 Routine: davinci_psc_all_enable
 Description:  Enable all PSC domains
**************************************/
void davinci_psc_all_enable(void)
{
        unsigned int alwaysOnPdNum = 0, dspPdNum = 1;
	unsigned int state;

	/**
	 * This function turns on all clocks in the ALWAYSON and DSP Power
	       * Domains.  Note this function assumes that the Power Domains are
	       * already on.
	       */

	/**
	 * For special workaround:  Clear MDCTL[i].EMURSTIE to 0x0 for
         * all of the following Modules. VPSSSLV, EMAC, EMACCTRL, MDIO,
         * USB, ATA, VLYNQ, HPI, DDREMIF, AEMIF, MMCSD, MEMSTICK, ASP,
	 * GPIO, IMCOP.
         */
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*5) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*5) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*6) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*6) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*7) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*7) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*13) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*13) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*14) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*14) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*19) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*19) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*18) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*18) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*26) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*26) | 0x003;
        *(volatile unsigned int*) (PSC_ADDR+0xA00+4*28) 
		= *(unsigned int*) (PSC_ADDR+0xA00+4*28) | 0x003;

	/**
	 * Set PTCMD.GO0 to 0x1 to initiate the state transtion for Modules
	 * in the ALWAYSON Power Domain
	       */
    	  *(volatile unsigned int*) PTCMD = (1<<alwaysOnPdNum);

    	  /* Wait for PTSTAT.GOSTAT0 to clear to 0x0 */
    	  while(! (((*(volatile unsigned int*) PTSTAT >> alwaysOnPdNum)
		    & 0x00000001) == 0));

	/**
	 * DO GEM AND IMCOP INITIALIZATION, ONLY IF DSP POWER DOMAIN IS OFF
         * NOTE: this is a precise and refined sequence 
	 * - use extreme care if modifying! 
	 */
        if ((PDSTAT1 & 0x1F) == 0)
        {
                /**
		 * set PSC FORCE mode; may not be necessary
		 */
            GBLCTL = GBLCTL | 0x01;     
             
            /* set DSP power domain next state to ON */     
            PDCTL1 = PDCTL1 | 0x01;     
      
            /* ensure external power indicator is cleared */     
            PDCTL1 = PDCTL1 & 0xFFFFFEFF;     
      
                // 1st step start
                /* SwRstDisable DSP module */
                *(volatile unsigned int*) (PSC_ADDR+0xA00+4*LPSC_GEM)
                        = (*(volatile unsigned int*)
                                (PSC_ADDR+0xA00+4*LPSC_GEM) & 0xFFFFFFF8);
      
            /* start power state transitions for DSP power domain */     
            *(volatile unsigned int*) PTCMD = (1<<dspPdNum);     
      
            /* wait for external power control pending to assert */     
                do{
                if (((EPCPR >> dspPdNum) & 0x00000001) == 1) {     
                                break;
            }     
                }while (1);
      
            /* close rail shorting switch */     
            CHP_SHRTSW = 0x1;     
      
            /* clear external power control pending register bit */     
            EPCCR = (1 << dspPdNum);     
      
                /* set external power good indicator */
                PDCTL1 = PDCTL1 | 0x0100;
  
            /* wait for DSP domain transitions to complete */     
                do {
                state = *(volatile unsigned int*) PTSTAT;     
                if (((state >> dspPdNum) & 0x00000001) == 0) {     
                                break;
                }     
                }while(1);

                // 2nd step start
                /* enable DSP module */
                *(volatile unsigned int*) (PSC_ADDR+0xA00+4*LPSC_GEM) =
                        (*(volatile unsigned int*)(PSC_ADDR+0xA00+4*LPSC_GEM)
			& 0xFFFFFFE0) | 0x3;

                /* hold DSP in reset on next power ON */
                *(volatile unsigned int*) (PSC_ADDR+0xA00+4*LPSC_GEM) =
                        *(volatile unsigned int*)
                                (PSC_ADDR+0xA00+4*LPSC_GEM) & 0xFFFFFEFF;

                /* set IMCOP to enable state */
                *(volatile unsigned int*) (PSC_ADDR+0xA00+4*LPSC_IMCOP) =
                        (*(volatile unsigned int*)(PSC_ADDR+0xA00+4*LPSC_IMCOP)
			& 0xFFFFFFE0) | 0x3;

                /* start power state transitions for DSP power domain */
                *(volatile unsigned int*) PTCMD = (1<<dspPdNum);

                /* wait for DSP domain transitions to complete */
                do {
                        state = *(volatile unsigned int*) PTSTAT;
                        if (((state >> dspPdNum) & 0x00000001) == 0) {
                                break;
            }     
                }while(1);
      
            /* turn off PSC FORCE mode */     
            GBLCTL = GBLCTL & 0xFFFFFFFE;     
    } /* END GEM AND IMCOP INITIALIZATION */ 

        /**
	 * Bringup UART out of reset here since NS16650 code
	 * that we are using from uBoot will not do it
         */
        *(volatile unsigned int*) UARTPWREMU_MGMT = 0x00006001;
        
	/* Enable GIO3.3V cells used for EMAC */
        *(volatile unsigned int*) VDD3P3V_PWDN = 0;

        /* Enable UART0 MUX lines */
        *(volatile unsigned int *)PINMUX4 |= 1;

        /* Enable EMAC and AEMIF pins */
#if (CONFIG_COMMANDS & CFG_CMD_NAND)
        *(volatile unsigned int*) PINMUX0 = 0x80000000;
#else
        *(volatile unsigned int*) PINMUX0 = 0x8000001F;
#endif

	/* Enable I2C pin Mux */
        *(volatile unsigned int *)PINMUX4 |= (1 << 7);

	/* Set the Bus Priority Register to highest possible value that
	   avoids starvation (0xFE). See Peripheral Bus Burst Priority Register
	   (PBBPR) in SPRUE22b, TMS320DM644x DMSoC DDR2 Memory Controller
	   User's Guide */
	/* The errata in SPRZ241F suggest that this value be set to 0x20 to avoid
	   possible bus priority inversion due to preference given to open banks,
	   regardless of the requesting module's priorities.
	   This may require tuning. */
	VBPR = 0x20;
}

/******************************
 Routine: disable_i2c_ddc()
 Description: disconnects the DDC portion of the I2C bus
******************************/
static void disable_i2c_ddc(void)
{
	/* Need to set the GPIO to high to disconnect DDC */
	GPIO_DDC_DISABLE_BANK->DIR &= ~GPIO_DDC_DISABLE_BIT;
	GPIO_DDC_DISABLE_BANK->SET |= GPIO_DDC_DISABLE_BIT;
	/* wait for the signal to be reflected in chip */
	udelay(2);
}

/******************************
 Routine: reset_i2c_scl()
 Description: unlocks the clock line on the I2C bus,
 necessary for I2C chips that do not have a reset line.
******************************/
static void reset_i2c_scl(void)
{
	int i;

	/* WARNING: this function is called before the serial port is
	 * initialized, CANNOT use printf or any console output !!! */

	/* To reset the THS7303 video filter it is necessary to create
	 * a clock cycle on the I2C bus, but the I2C controller cannot
	 * do that since I2C SCL may be held low by the THS7303. We
	 * use the SCL pin in GPIO mode to simulate a clock cycle. */

	/* The SCL line should be operated in open-drain fashion,
	 * so we never force it to "1". */

	/* set I2C clock pin to GPIO */
        *(volatile unsigned int *)PINMUX4 &= ~PINMUX4_I2CEN;

	/* Allow for devices to quiesce, we may have just received power */
	for (i = 0; i < 20; i++)
		udelay(1000);

	/* be sure SDA pin is in input direction */
	GPIOBK23->DIR |= GPIO_I2C_SDA_BIT;

	/* be sure SCL pin is in input direction (i.e. high-Z) */
	GPIOBK23->DIR |= GPIO_I2C_SCL_BIT;
	/* default SCL to low when switched to ouput direction */
	GPIOBK23->OUT &= ~GPIO_I2C_SCL_BIT;

	/* put cycles on SCL as long as SDA is being held low */
	i2c_bus_count = 0;
	i2c_bus_failed = 0;
	while ( ( GPIOBK23->IN & GPIO_I2C_SDA_BIT ) == 0 )
	{
	    if (i2c_bus_count++ > 10) {
		i2c_bus_failed = 1;
		break;
	    }
	    /* SCL pin is on high-Z */
	    udelay(71);
	    /* set SCL output direction: drives SCL pin low */
	    GPIOBK23->DIR &= ~GPIO_I2C_SCL_BIT;
	    udelay(71);
	    /* set SCL input direction: SCL on high-Z */
	    GPIOBK23->DIR |= GPIO_I2C_SCL_BIT;
	    udelay(470);
	}

	/* set I2C clock pin back to I2C */
        *(volatile unsigned int *)PINMUX4 |= PINMUX4_I2CEN;
}

/******************************
 Routine: reset_peripherals()
 Description: resets all the peripherals connected to the reset line
******************************/
static void reset_peripherals(void)
{
	int i;

	GPIO_PERIPH_RESET_BANK->DIR &=
		~GPIO_PERIPH_RESET_BIT; /* set output direction */
	GPIO_PERIPH_RESET_BANK->CLR |=
		GPIO_PERIPH_RESET_BIT; /* set line low */

	for (i = 0; i < 20; i++) /* wait the number of ms */
		udelay(1000);

	GPIO_PERIPH_RESET_BANK->SET |=
		GPIO_PERIPH_RESET_BIT; /* set line high */
	GPIO_PERIPH_RESET_BANK->DIR |=
		GPIO_PERIPH_RESET_BIT; /* set input direction */

	for (i = 0; i < 20; i++) /* wait the number of ms */
		udelay(1000);
}

#define CDCE_I2C_ADDR		0x69
#define CDCE_CMD_BYTE_SEL_BIT	0x80
#define CDCE_PLL_SEL_P5_REG	12
#define CDCE_PLL_SEL_P5_MASK	( ( ( 1 << 3 ) - 1 ) << 3 )
#define CDCE_PLL_SEL_P5_SHIFT	3
#define CDCE_P_PLL_BYPASS	0
#define CDCE_P5_REG		18
#define CDCE_Y2_REG		21
#define CDCE_Y5_REG		24
#define CDCE_Y_DIV_SEL_MASK	( ( 1 << 3 ) - 1 )
#define CDCE_Y_ENABLE		( 1 << 3 )
#define CDCE_Y_SLEW_SLOWEST	( 3 << 4 )
#define CDCE_Y_DIV_SEL_P5	5

#ifdef BONSAI_CDCE_BYPASS
/******************************
 Routine: CDCEsetup
 Description:  Sets up the CDCE so that the 27 MHz output bypasses the PLLs
******************************/
static void CDCEsetup(void)
{
	uchar rval;

	/* turn off audio clock on Y2, may conflict with default audio clock
	 * pin setting on DaVinci. */
	rval = i2c_reg_read(CDCE_I2C_ADDR,
			    CDCE_CMD_BYTE_SEL_BIT | CDCE_Y2_REG);
	rval &= ~(CDCE_Y_ENABLE);
	i2c_reg_write(CDCE_I2C_ADDR,
		      CDCE_CMD_BYTE_SEL_BIT | CDCE_Y2_REG, rval);

	/* NOTE: as we are running on the 27 MHz clock we need to
	 * switch the output in one step, so we prepare the output
	 * from a free divider first and then we switch. */

	/* We set post-divider P5 for the system clock source (output
	 * Y5) and P5's source to bypass the PLLs.  */

	rval = i2c_reg_read(CDCE_I2C_ADDR,
			    CDCE_CMD_BYTE_SEL_BIT | CDCE_Y5_REG);
	if ((rval & CDCE_Y_DIV_SEL_MASK) == CDCE_Y_DIV_SEL_P5)
		return; /* the output is already set to P5, all is OK */

	printf("INFO: reconfiguring CDCE for 27 MHz PLL-bypass...");

	/* make P5 post-divisor = 1 */
	i2c_reg_write(CDCE_I2C_ADDR, CDCE_CMD_BYTE_SEL_BIT | CDCE_P5_REG, 1);

	/* make P5 source = PLL bypass */
	rval = i2c_reg_read(CDCE_I2C_ADDR,
			    CDCE_CMD_BYTE_SEL_BIT | CDCE_PLL_SEL_P5_REG);
	rval &= ~(CDCE_PLL_SEL_P5_MASK);
	rval |= (CDCE_P_PLL_BYPASS << CDCE_PLL_SEL_P5_SHIFT);
	i2c_reg_write(CDCE_I2C_ADDR,
		      CDCE_CMD_BYTE_SEL_BIT | CDCE_PLL_SEL_P5_REG, rval);

	/* make Y5 source be P5 */
	i2c_reg_write(CDCE_I2C_ADDR, CDCE_CMD_BYTE_SEL_BIT | CDCE_Y5_REG,
		      CDCE_Y_DIV_SEL_P5 | CDCE_Y_ENABLE |
		      CDCE_Y_SLEW_SLOWEST);

	printf("done\n");
}
#endif /* BONSAI_CDCE_BYPASS */

#ifdef BONSAI_TPS_WORKAROUND
/******************************
 Routine: TPSsetup
 Description:  Change VDCDC1 value for stabilizing power system.
******************************/
#define TPS_I2C_ADDR         0x48
#define TPS_I2C_LEN      9
#define TPS_CONTROL2     0x05
#define GO               0x80
#define NOADJ            0x40
#define TPS_DEFCORE      0x06

static void TPSsetup(void)
{
     int i;
     unsigned char tpsbuf[TPS_I2C_LEN];
   
     printf("WARNING: setting VDCDC1 to 1.25 V\n");
     for(i=0; i<TPS_I2C_LEN; i++) {
	  tpsbuf[i] = i2c_reg_read(TPS_I2C_ADDR, (uchar)i);
     }

     tpsbuf[TPS_CONTROL2] &= ~NOADJ;
     i2c_reg_write(TPS_I2C_ADDR, TPS_CONTROL2, tpsbuf[TPS_CONTROL2]);

     tpsbuf[TPS_DEFCORE] = 0x12; // VDCDC1 = 1.25 V
     i2c_reg_write(TPS_I2C_ADDR, TPS_DEFCORE, tpsbuf[TPS_DEFCORE]);

     tpsbuf[TPS_CONTROL2] |= GO;
     i2c_reg_write(TPS_I2C_ADDR, TPS_CONTROL2, tpsbuf[TPS_CONTROL2]);

     tpsbuf[TPS_CONTROL2] |= NOADJ;
     i2c_reg_write(TPS_I2C_ADDR, TPS_CONTROL2, tpsbuf[TPS_CONTROL2]);
}
#endif /* BONSAI_TPS_WORKAROUND */

/******************************
 Routine: vid_filt_reset()
 Description: Video filter reset, must be called before any other
 operation on I2C bus
******************************/
#define VID_FILT_I2C_ADDR 0x2C

static void vid_filt_reset(void)
{
	uchar val;
	int i;

	/* Setup VGA filter to off */
	val = 0;
	for (i = 1; i <= 3; i++) {
		if ( i2c_write(VID_FILT_I2C_ADDR, i, 1, &val, 1) ) {
			printf("WARNING: failed resetting video filter (not "
			       "present?)\n");
			return;
		}
	}
}

/******************************
 Routine: vga_powerdown()
 Description: Powerdowns the VGA encoder chip, which starts in powerup mode.
******************************/
#define VGA_I2C_ADDR			0x20
#define THS_CHIPCTL_REG			0x03
#define THS_CHIPCTL_DAC_PWDN		(1 << 3)
#define THS_CHIPCTL_CHIP_PWDN		(1 << 2)

static void vga_powerdown(void)
{
	uchar rval = ( THS_CHIPCTL_DAC_PWDN | THS_CHIPCTL_CHIP_PWDN );
	i2c_reg_write(VGA_I2C_ADDR, THS_CHIPCTL_REG, rval);
}

/******************************
 Routine: vdec_powerdown()
 Description: Powerdowns the video decoder chip, which starts in powerup mode.
******************************/

#define VDEC_I2C_ADDR			0x5C
#define VDEC_OPMODE_CTRL_REG		0x02
#define VDEC_OPMODE_CTRL_PWDN		(1 << 0)

static void vdec_powerdown(void)
{
	if ( carrier_hw_rev >= MK_HW_REV(2,14) || i2c_probe(VDEC_I2C_ADDR) )
		return; /* new design without video decoder */
	i2c_reg_write(VDEC_I2C_ADDR, VDEC_OPMODE_CTRL_REG,
		      VDEC_OPMODE_CTRL_PWDN);
}

/******************************
 Routine: hdmi_powerdown()
 Description: Powerdowns the HDMI encoder chip; it start in power down by
 default but it may stay in powerup after a reset.
******************************/

#define HDMI_I2C_ADDR		0x3D
#define AD9889_SYS_PD_REG	0x41
#define AD9889_SYS_PD_BIT	( 1 << 6 )

static void hdmi_powerdown(void)
{
	uchar rval;
	rval = i2c_reg_read(HDMI_I2C_ADDR, AD9889_SYS_PD_REG);
	rval |= AD9889_SYS_PD_BIT;
	i2c_reg_write(HDMI_I2C_ADDR, AD9889_SYS_PD_REG, rval);
}

/******************************
 Routine: usb_power_off()
 Description: Turns off power to USB peripherals.
******************************/
static void usb_power_off(void)
{
	/* NOTE: setting to 'on' depends on hw revision, either GPIO
	 * pin to 'Z' or '1'; but setting to 'off' just requires to
	 * output '0' */
	GPIO_USBPWR_BANK->CLR |= GPIO_USBPWR_BIT; /* set to low */
	GPIO_USBPWR_BANK->DIR &= ~GPIO_USBPWR_BIT; /* set output direction */
}

/******************************
 Routine: make_id()
 Description:  creates a ID number string from the raw HW info
 The destination string should be long enough (32 bytes OK).
******************************/
static void make_id(char* id_string, const unsigned char *hw_info)
{
	u32 id_nr;
	int i;
	memcpy(id_string, hw_info, 21);
	/* protect against non-printable chars */
	for (i = 0; i < 21; i++) {
	    if (id_string[i] <= ' ' || id_string[i] > '~')
		id_string[i] = '?';
	}
	/* memcpy below is needed because of alignment constraints */
	memcpy(&id_nr, hw_info + 21, 4);
	id_nr = ntohl(id_nr);
	sprintf(id_string+21, "%04u", id_nr);
	return;
}

/******************************
 Routine: init_hw_revision()
 Description:  Parses the HW info data to extract the revision number
******************************/
static int init_hw_revision(unsigned char *hw_info, u32 *rev)
{
	char number_string[3];
	char *p;
	int major, minor;

	if (hw_info[0] == 0xFF && hw_info[1] == 0xFF && hw_info[2] == 0xFF)
	    return -1; /* HW info not set */
	if (strncmp(hw_info, "SPX", 3) != 0) {
		printf("Invalid signature in HW info\n");
		return 1;
	}

	memcpy(number_string, hw_info+11, 2);
	number_string[2] = '\0';
	major = simple_strtol(number_string, &p, 10);
	if (*p != '\0' || number_string[0] == '\0') {
		printf("ERROR: invalid major revision number in HW info\n");
		return 1;
	}
	memcpy(number_string, hw_info+14, 2);
	number_string[2] = '\0';
	minor = simple_strtol(number_string, &p, 10);
	if (*p != '\0' || number_string[0] == '\0') {
		printf("ERROR: invalid minor revision number in HW info\n");
		return 1;
	}
	*rev = MK_HW_REV(major, minor);
	return 0;
}

/******************************
 Routine: load_hw_info()
 Description:  loads the hw info from EEPROMs, but does not parse it,
 except for the HW revision numbers.
******************************/
static void load_hw_info(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	uchar eeprom_core_addr;
	int r;

	if (i2c_bus_failed)
		return;
	if ( ! i2c_probe(EEPROM_CORE_I2C_ADDR) ) {
		eeprom_core_addr = EEPROM_CORE_I2C_ADDR;
	} else if ( ! i2c_probe(EEPROM_CORE_I2C_ADDR_OLD) ) {
		eeprom_core_addr = EEPROM_CORE_I2C_ADDR_OLD;
	} else {
		printf("ERROR: no core EEPROM detected\n");
		return;
	}

	if (i2c_read(eeprom_core_addr, 0x00, 1,
		     core_hw_info, sizeof(core_hw_info)) != 0) {
		printf("ERROR: failed reading core EEPROM\n");
	} else {
		core_hw_info_ok = -1;
		r = init_hw_revision(core_hw_info, &core_hw_rev);
		if (r < 0) {
			printf("WARNING: core EEPROM not initialized\n");
		} else if (r > 0) {
			printf("ERROR: invalid signature / revision numbers "
			       "in core EEPROM\n");
		} else {
		    core_hw_info_ok = 1;
		}
	}
	if (i2c_read(EEPROM_CARRIER_I2C_ADDR, 0x00, 1,
		     carrier_hw_info, sizeof(carrier_hw_info)) != 0) {
		printf("ERROR: failed reading carrier EEPROM\n");
	} else {
		carrier_hw_info_ok = -11;
		r = init_hw_revision(carrier_hw_info, &carrier_hw_rev);
		if (r < 0) {
			printf("WARNING: carrier EEPROM not initialized\n");
		} else if (r > 0) {
			printf("ERROR: invalid signature / revision numbers "
			       "in carrier EEPROM\n");
		} else {
		    carrier_hw_info_ok = 1;
		}
	}
	if ( (core_hw_info_ok < 0 || carrier_hw_info < 0) &&
	     ! (gd->flags & GD_FLG_DEFENV) )
		printf("WARNING: HW info not in EEPROM but U-Boot "
		       "environment set, should erase it and reboot\n");
}

/******************************
 Routine: parse_hw_info()
 Description:  parses the previously loaded hw info
******************************/

static int parse_hw_info(void)
{
	extern flash_info_t flash_info[];
	const unsigned char *emac;
	char emac_string[18];
	char id_string[64];
	char serial_string[17];
	char addr_string[16];
	char *p;
	int i;
	u32 tmp;
	if (!core_hw_info_ok || !carrier_hw_info_ok) {
		printf("ERROR: did not read HW info from EEPROMs\n");
		return 1;
	}
	if (core_hw_info_ok > 0 && strncmp(core_hw_info, "SPX", 3) != 0) {
		printf("ERROR: invalid signature in core EEPROM\n");
		return 1;
	}
	if (carrier_hw_info_ok > 0 &&
	    strncmp(carrier_hw_info, "SPX", 3) != 0) {
		printf("ERROR: invalid signature in carrier EEPROM\n");
		return 1;
	}

	/* Core ID number */
	if (core_hw_info_ok > 0)
		make_id(id_string, core_hw_info);
	else
		strcpy(id_string, "<not set>");
	printf("Core ID #        : %s\n", id_string);
	printf("Core revision    : %i.%i\n",
	       HW_REV_MAJOR(core_hw_rev), HW_REV_MINOR(core_hw_rev));
	/* Carrier ID number */
	if (carrier_hw_info_ok > 0)
		make_id(id_string, carrier_hw_info);
	else
		strcpy(id_string, "<not set>");
	printf("Carrier ID #     : %s\n", id_string);
	printf("Carrier revision : %i.%i\n",
	       HW_REV_MAJOR(carrier_hw_rev), HW_REV_MINOR(carrier_hw_rev));
	if ( (core_hw_rev == 0 && core_hw_info_ok > 0) ||
	     (carrier_hw_rev == 0 && carrier_hw_info_ok > 0) ) {
		printf("ERROR: invalid hardware revision number(s)\n");
		return 1;
	}
	/* Check EMAC address */
	emac = NULL;
	if (carrier_hw_info_ok > 0) {
		emac = carrier_hw_info + ETH_ADDR_OFFSET;
		for (i = 0; i < 6 && (emac[i]&0xFF) == 0xFF; i++)
			;
		if (i == 6) {
			printf("WARNING: Ethernet MAC address not set in "
			       "EEPROM\n");
		}
	}
	if (emac) {
		/* Convert EMAC to nul-terminated string */
		for (i = 0, p = emac_string; i < 6; i++) {
			unsigned d;
			d = (emac[i] >> 4);
			*(p++) = (d <= 9) ? ('0' + d) : ('a' + (d-10));
			d = (emac[i] & 0xF);
			*(p++) = (d <= 9) ? ('0' + d) : ('a' + (d-10));
			if (i < 5)
				*(p++) = ':';
		}
		*(p++) = 0;
		if (getenv("ethaddr") == NULL ||
		    strcmp(getenv("ethaddr"), emac_string) != 0)
			setenv("ethaddr", emac_string);
	} else {
		strcpy(emac_string, "<not set>");
	}
	printf("Ethernet address : %s\n", emac_string);
	/* Make board serial number, same as MAC address */
	if (emac != NULL) {
		memcpy(&tmp, emac, sizeof(tmp));
		board_serial_nr.high = (ntohl(tmp) >> 16) & 0xFFFF;
		memcpy(&tmp, emac+2, sizeof(tmp));
		board_serial_nr.low = ntohl(tmp);
		sprintf(serial_string, "%08x%08x",
			board_serial_nr.high, board_serial_nr.low);
		if (getenv("serial#") == NULL)
			setenv("serial#", serial_string);
	} else {
		strcpy(serial_string, "<not set>");
	}
	printf("Serial #         : %s\n", serial_string);
	/* set the board revision as environment variable */
	if ( carrier_hw_rev ) {
		sprintf(addr_string, "0x%08x", carrier_hw_rev);
		setenv("boardrev", addr_string);
	} else {
		setenv("boardrev", NULL);
	}
	/* set the addresses as environment variables */
	sprintf(addr_string, "0x%08x", FLASH_ADDR_KERNEL1);
	setenv("addrmainkrnl", addr_string);
	sprintf(addr_string, "0x%08x", FLASH_ADDR_KERNEL2);
	setenv("addrsafekrnl", addr_string);
	sprintf(addr_string, "0x%08x", FLASH_ADDR_INITRD2);
	setenv("addrsafedisk", addr_string);
	sprintf(addr_string, "0x%08x",
		FLASH_ADDR_KERNEL2 - FLASH_ADDR_KERNEL1);
	setenv("sizemainkrnl", addr_string);
	sprintf(addr_string, "0x%08x",
		FLASH_ADDR_INITRD2 - FLASH_ADDR_KERNEL2);
	setenv("sizesafekrnl", addr_string);
	sprintf(addr_string, "0x%08x",
		FLASH_ADDR_SAFEDATA - FLASH_ADDR_INITRD2);
	setenv("sizesafedisk", addr_string);
	sprintf(addr_string, "0x%08x",
		flash_info[0].start[0] + flash_info[0].size -
		FLASH_ADDR_SAFEDATA);
	setenv("sizesafedata", addr_string);
	return 0;
}

/******************************
 Routine: check_image
 Description:  Checks that an image at 'addr' is correct.
******************************/
static int check_image(ulong addr, int header_only)
{
	/* mostly copied from image_info() in cmd_bootm.c */
	ulong	data, len, checksum;
	image_header_t header;
	image_header_t *hdr = &header;

	/* Copy header so we can blank CRC field for re-calculation */
	memmove (&header, (char *)addr, sizeof(image_header_t));

	if (ntohl(hdr->ih_magic) != IH_MAGIC)
		return 1; /* bad magic number */

	data = (ulong)&header;
	len  = sizeof(image_header_t);

	checksum = ntohl(hdr->ih_hcrc);
	hdr->ih_hcrc = 0;

	if (crc32 (0, (char *)data, len) != checksum)
		return 1; /* bad header checksum */

	if (header_only)
		return 0; /* OK as far as we tested */

	data = addr + sizeof(image_header_t);
	len  = ntohl(hdr->ih_size);

	if (crc32 (0, (char *)data, len) != ntohl(hdr->ih_dcrc))
		return 1; /* bad data checksum */

	return 0; /* OK */
}

/******************************
 Routine: check_preboot
 Description:  performs all preboot checks and sets 'preboot' is not passed
******************************/
static void check_preboot(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	int preboot_ok = 1;
	char *p;

	/* check failsafe and initrd images */
	if ( check_image(FLASH_ADDR_KERNEL2, 1) ||
	     check_image(FLASH_ADDR_INITRD2, 1) )
		preboot_ok = 0;

	/* check that environment is OK */
	if (gd->flags & GD_FLG_DEFENV)
		preboot_ok = 0;

	/* check that EEPROMs have valid data */
	if (core_hw_info_ok < 0 || carrier_hw_info_ok < 0)
		preboot_ok = 0;

	/* check that we have a MAC address / serial number */
	if ( getenv("ethaddr") == NULL ||
	     (board_serial_nr.high == 0 && board_serial_nr.low == 0) )
		preboot_ok = 0;

	p = getenv("needsetup");
	if (!p || strcmp(p, "never") != 0)
		setenv("needsetup", (preboot_ok ? "no" : "yes"));

	if (check_pushbtn(CFG_BONSAI_RECOVERY_WAIT)) {
		extern flash_info_t flash_info[];
		int s_first, s_last;

		printf("INFO: system configuration reset activated "
		       "by push button\n");
		setenv("recovery", "yes");

		printf("Erasing failsafe variable data\n");
		s_first = ( FLASH_ADDR_SAFEDATA - flash_info[0].start[0] ) /
			CFG_FLASH_SECT_SZ;
		s_last = flash_info[0].size / CFG_FLASH_SECT_SZ - 1;

		if ( flash_erase(flash_info, s_first, s_last) )
			printf("Failed!");
	} else {
		setenv("recovery", "no");
	}
}

/******************************
 Routine: misc_init_r
 Description:  Misc. init
******************************/
int misc_init_r (void)
{
        int clk = 0;

	DECLARE_GLOBAL_DATA_PTR;

#ifdef CONFIG_STATUS_LED
	status_led_set(STATUS_LED_GREEN, STATUS_LED_ON);
#endif

	/* The video filter may hold the I2C bus while not reset, so
	 * reset it before trying to access anything else on I2C
	 * (e.g., EEPROMs) */
	vid_filt_reset();

	load_hw_info();

#ifdef CONFIG_SILENT_CONSOLE
	if ( carrier_hw_rev > MK_HW_REV(2,1) &&
	    ( ! check_pushbtn(CFG_BONSAI_SILENT_WAIT) ) ) {
		/* nothing to do, silent mode is the default */
	} else {
		/* revert default silent mode to normal mode */
		const char* v;
		gd->flags &= ~GD_FLG_SILENT;
		setenv("stdin", "serial");
		setenv("stdout", "serial");
		setenv("stderr", "serial");
		if ( (v = getenv("ver")) )
			printf ("\n\n%s\n\n", v);
	}
#endif

        clk = ((PLL2_PLLM + 1) * 27) / (PLL2_DIV2 + 1);

        printf ("ARM Clock :- %dMHz\n", ((((PLL1_PLLM + 1) * 27 ) / 2)) );
        printf ("DDR Clock :- %dMHz\n", (clk/2));

	printf ("PLL1: M = %u, D1 = %u, D2 = %u, D3 = %u, D5 = %u\n",
		(PLL1_PLLM + 1), (PLL1_DIV1 + 1),
		(PLL1_DIV2 + 1), (PLL1_DIV3 + 1),
		(PLL1_DIV5 + 1));

	printf ("PLL2: M = %u, D1 = %u, D2 = %u\n",
		(PLL2_PLLM + 1), (PLL2_DIV1 + 1),
		(PLL2_DIV2 + 1));

	/* check that board_init went well and report any errors */

	if (i2c_bus_count > 1)
		printf ("I2C bus SCL count = %d\n", i2c_bus_count);
	if (i2c_bus_failed)
		panic("PANIC: I2C bus is locked... rebooting\n");
	if ( ! core_hw_info_ok ) {
		panic("PANIC: failed reading core EEPROM... rebooting\n");
	} else if ( ! core_hw_rev && core_hw_info_ok >= 0 ) {
		panic("PANIC: invalid revision numbers in "
		      "core EEPROM... rebooting\n");
	}
	if ( ! carrier_hw_info_ok ) {
		panic("PANIC: failed reading carrier EEPROM... rebooting\n");
	} else if ( ! carrier_hw_rev && carrier_hw_info_ok >= 0 ) {
		panic("PANIC: invalid revision numbers in "
			"carrier EEPROM... rebooting\n");
	}

	usb_power_off();
	vga_powerdown();
	vdec_powerdown();
	hdmi_powerdown();

#ifdef BONSAI_CDCE_BYPASS
	if (core_hw_rev != 0 && core_hw_rev <= MK_HW_REV(1,0))
		CDCEsetup();
#endif

#ifdef BONSAI_TPS_WORKAROUND
	TPSsetup();
#endif

	if (getenv("mem") == NULL) {
		char mem_string[32];
		sprintf(mem_string, "%ldk", LINUX_MEM / 1024);
		setenv("mem", mem_string);
	}

	setenv("build", BUILD_TAG);

	if (parse_hw_info())
		panic("PANIC: failed parsing HW info block\n");

#if (CONFIG_COMMANDS & CFG_CMD_DATE)
	run_command("date", 0);
#endif

#ifdef CONFIG_PREBOOT
	/* Make sure we always have the compiled-in preboot setup */
	setenv("preboot", CONFIG_PREBOOT);
#endif

	/* If the bootcmd is gone for some reason use the compiled-in one */
	if (getenv("bootcmd") == NULL)
	    setenv("bootcmd", CONFIG_BOOTCOMMAND);

	check_preboot();

	printf("\n"
	       "*************************************************\n"
	       "*                                               *\n"
	       "*               !!! WARNINNG !!!                *\n"
	       "*                                               *\n"
	       "*   Modifying the U-boot environment variables  *\n"
	       "*   may make this device to malfunction. Do     *\n"
	       "*   not modify the environment unless you know  *\n"
	       "*   what you are doing !                        *\n"
	       "*                                               *\n"
	       "*************************************************\n"
	       "\n");

	return 0;
}

/******************************
 Routine: dram_init
 Description:  Memory Info
******************************/
int dram_init (void)
{
        DECLARE_GLOBAL_DATA_PTR;
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;
        return 0;
}

#if (CONFIG_COMMANDS & CFG_CMD_NAND)
void nand_init(void)
{
    extern flash_info_t flash_info[];

    nand_probe(CFG_NAND_ADDR);
	if (nand_dev_desc[0].ChipID != NAND_ChipID_UNKNOWN) 
	{
                print_size(nand_dev_desc[0].totlen, "\n");
    }
}
#endif

#ifdef CONFIG_BOOTCOUNT_LIMIT

void bootcount_store (ulong a)
{
	volatile ulong *save_addr =
		(volatile ulong *)( BOOTCOUNT_ADDR );

	save_addr[0] = BOOTCOUNT_MAGIC;
	save_addr[1] = a;
}

ulong bootcount_load (void)
{
	volatile ulong *save_addr =
		(volatile ulong *)( BOOTCOUNT_ADDR );

	if (save_addr[0] != BOOTCOUNT_MAGIC)
		return 0;
	else
		return save_addr[1];
}

#endif /* CONFIG_BOOTCOUNT_LIMIT */
